@@ -441,6 +441,7 @@ class ApplicationSession(BaseSession):
                     self._realm = msg.realm
 
                 self._session_id = msg.session
+                self._router_roles = msg.roles
 
                 details = SessionDetails(self._realm, self._session_id, msg.authid, msg.authrole, msg.authmethod, msg.authprovider, msg.authextra)
                 # firing 'join' *before* running onJoin, so that the
@@ -587,7 +588,18 @@ class ApplicationSession(BaseSession):
                         invoke_kwargs = msg.kwargs if msg.kwargs else dict()
 
                         if handler.details_arg:
-                            invoke_kwargs[handler.details_arg] = types.EventDetails(publication=msg.publication, publisher=msg.publisher, publisher_authid=msg.publisher_authid, publisher_authrole=msg.publisher_authrole, topic=topic, enc_algo=msg.enc_algo)
+                            invoke_kwargs[handler.details_arg] = types.EventDetails(publication=msg.publication, publisher=msg.publisher, publisher_authid=msg.publisher_authid, publisher_authrole=msg.publisher_authrole, topic=topic, retained=msg.retained, enc_algo=msg.enc_algo)
+
+                        # FIXME: https://github.com/crossbario/autobahn-python/issues/764
+                        def _success(_):
+                            # Acknowledged Events -- only if we got the details header and
+                            # the broker advertised it
+                            if msg.x_acknowledged_delivery and self._router_roles["broker"].x_acknowledged_event_delivery:
+                                if self._transport:
+                                    response = message.EventReceived(msg.publication)
+                                    self._transport.send(response)
+                                else:
+                                    self.log.warn("successfully processed event with acknowledged delivery, but could not send ACK, since the transport was lost in the meantime")
 
                         def _error(e):
                             errmsg = 'While firing {0} subscribed under {1}.'.format(
@@ -595,7 +607,7 @@ class ApplicationSession(BaseSession):
                             return self._swallow_error(e, errmsg)
 
                         future = txaio.as_future(handler.fn, *invoke_args, **invoke_kwargs)
-                        txaio.add_callbacks(future, None, _error)
+                        txaio.add_callbacks(future, _success, _error)
 
                 else:
                     raise ProtocolError("EVENT received for non-subscribed subscription ID {0}".format(msg.subscription))
